/*
 * ar2c-lp-utils - ar2c_parser.h
 * 	version 0.1.1
 *
 * 	AR-2c protocol messages definition.
 * 	
 * Copyright (C) 2004..2006  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef AR2C_PARSER_INCLUDED
#define AR2C_PARSER_INCLUDED

#include <inttypes.h>

#ifdef HAVE_BYTESWAP_H
#include <byteswap.h>
#else

static inline uint32_t bswap_32(uint32_t x)
{
	return ((x >> 24) | ((x >> 8) & 0xff00) | ((x << 8) & 0xff0000)
			| ((x << 24) & 0xff000000));
}

#endif

#define AR2C_LPIF_WYSREJ		0x01	/* WYSREJ extensions */
#define AR2C_LPIF_SPDSET		0x02	/* speed setting extensions */
#define AR2C_LPIF_MODESW		0x04	/* mode switch extenstions */
#define AR2C_LPIF_PRNCTRL		0x08	/* printer ctrl extensions */
#define AR2C_LPIF_REJINFO		0x10	/* rej info extenstions */
#define AR2C_LPIF_CF_SET		0x20	/* current flags setting */
#define AR2C_LPIF_CRYPTO		0x100	/* lp_crypto extensions */

struct ar2c_lp_info {
	uint8_t protocol_version;	/* 3 or 4 */
	uint8_t model_id;		/* 2 for AR-2c or 3 for AR-3c */
	uint16_t len;			/* info lenght */
	uint32_t software_version;	/* model specific */
	uint32_t supported_flags;	/* supported feature flags */
	uint32_t current_flags;		/* enabled feature flags */
	uint8_t description[48];	/* model specific description */
};

union ar2c_lp_v3_analog_msg {
	uint32_t value;
	struct {
		uint32_t value:8;
		uint32_t sign:1;
		uint32_t ts:10;
		uint32_t id:4;
		uint32_t magic:1;	// 0
		uint32_t pad0:8;
	} bits;
};

union ar2c_lp_v3_binary_msg {
	uint32_t value;
	struct {
		uint32_t value:1;
		uint32_t ts:13;
		uint32_t id:6;
		uint32_t magic:4;	// 0x8
		uint32_t pad0:8;
	} bits;
};

union ar2c_lp_v3_binarys_msg {
	uint32_t value;
	struct {
		uint32_t values:16;
		uint32_t id:4;
		uint32_t magic:4;	// 0x9
		uint32_t pad0:8;
	} bits;
};

union ar2c_lp_v3_bell_msg {
	uint32_t value;
	struct {
		uint32_t values:4;
		uint32_t ts:13;
		uint32_t reserved0:3;
		uint32_t magic:4;	// 0xA
		uint32_t pad0:8;
	} bits;
};

union ar2c_lp_v3_hbs_msg {
	uint32_t value;
	struct {
		uint32_t id:8;
		uint32_t ts:8;
		uint32_t reserved0:4;
		uint32_t magic:4;	// 0xd
		uint32_t pad0:8;
	} bits;
};

union ar2c_lp_v3_pos_msg {
	uint32_t value;
	struct {
		uint32_t value:16;
		uint32_t sign:1;
		uint32_t reserved0:3;
		uint32_t magic:4;	// 0xB
		uint32_t pad0:8;
	} bits;
};

union ar2c_lp_v3_min_chg_msg {
	uint32_t value;
	struct {
		uint32_t min:8;	// BCD
		uint32_t hour:6;	// BCD
		uint32_t mday:6;	// BCD
		uint32_t magic:4;	// 0xe
		uint32_t pad0:8;
	} bits;
};

union ar2c_lp_v3_date_chg_msg {
	uint32_t value;
	struct {
		uint32_t day:6;	// BCD
		uint32_t mon:6;	// BCD
		uint32_t year:8;	// BCD
		uint32_t magic:4;	// 0xf
		uint32_t pad0:8;
	} bits;
};

union ar2c_lp_v3_lchg_msg {
	uint32_t value;
	struct {
		uint32_t layer:8;
		uint32_t reserved:8;
		uint32_t magic:8;	// 0xff
		uint32_t pad0:8;
	} bits;
};

/*
 * FIXME: Do it big endian safe(?).
 */

union ar2c_lp_v4_analog_msg {
	uint32_t value;
	struct {
		int32_t value:10;
		uint32_t ts:13;
		uint32_t id:8;
		uint32_t magic:1;	// 0
	} bits;
};

union ar2c_lp_v4_binary_msg {
	uint32_t value;
	struct {
		uint32_t value:1;
		uint32_t ts:16;
		uint32_t id:11;
		uint32_t magic:4;	// 0x8
	} bits;
};
	
union ar2c_lp_v4_binarys_msg {
	uint32_t value;
	struct {
		uint32_t values:16;
		uint32_t id:12;	// up to 2^28 binary inputs ??
		uint32_t magic:4;	// 0x9 
	} bits;
};

/*
 * FIXME: ts & id are quessed.
 */

union ar2c_lp_v4_bell_msg {
	uint32_t value;
	struct {
		uint32_t id:12;
		uint32_t ts:16;
		uint32_t magic:4;	// 0xA
	} bits;
};

/*
 * FIXME: Fix ar2c_lp_v4_pos_msg.
 */

union ar2c_lp_v4_pos_msg {
	uint32_t value;
	struct {
		uint32_t value:17;
		uint32_t sign:1;
		uint32_t ts:10;
		uint32_t magic:4;	// 0xB
	} bits;
};

union ar2c_lp_v4_min_chg_msg {
	uint32_t value;
	struct {
		uint32_t min:8;	// BCD
		uint32_t hour:6;	// BCD
		uint32_t mday:6;	// BCD
		uint32_t mon:6;	// BCD
		uint32_t reserved0:2;	// 0
		uint32_t magic:4;	// 0xe
	} bits;
};

union ar2c_lp_v4_date_chg_msg {
	uint32_t value;
	struct {
		uint32_t day:6;	// BCD
		uint32_t mon:6;	// BCD
		uint32_t year:16;	// BCD
		uint32_t magic:4;	// 0xf
	} bits;
};

#define AR2C_LP_V3_ANALOG_MSG_INITIALIZER { .bits = { 0, 0, 0, 0, 0, }, }
#define AR2C_LP_V3_BINARY_MSG_INITIALIZER { .bits = { 0, 0, 0, 0x8, }, }
#define AR2C_LP_V3_BINARY_HI_MSG_INITIALIZER { .bits = { 0, 0, 0, 0xc, }, }
#define AR2C_LP_V3_BINARYS_MSG_INITIALIZER { .bits = { 0, 0, 0x9, }, }
#define AR2C_LP_V3_BELL_MSG_INITIALIZER { .bits = { 0, 0, 0, 0xa, }, }
#define AR2C_LP_V3_HBS_MSG_INITIALIZER { .bits = { 0, 0, 0, 0xd, }, }
#define AR2C_LP_V3_POS_MSG_INITIALIZER { .bits = { 0, 0, 0xb, }, }
#define AR2C_LP_V3_MIN_CHG_MSG_INITIALIZER { .bits = { 0, 0, 0, 0xe, }, }
#define AR2C_LP_V3_DATE_CHG_MSG_INITIALIZER { .bits = { 0, 0, 0, 0xf, }, }
#define AR2C_LP_V3_LCHG_MSG_INITIALIZER { .bits = { 0, 0, 0xff, }, }

#define AR2C_LP_V4_ANALOG_MSG_INITIALIZER { .bits = { 0, 0, 0, 0, }, }
#define AR2C_LP_V4_BINARY_MSG_INITIALIZER { .bits = { 0, 0, 0, 0x8, }, }
#define AR2C_LP_V4_BINARYS_MSG_INITIALIZER { .bits = { 0, 0, 0x9, }, }
#define AR2C_LP_V4_BELL_MSG_INITIALIZER { .bits = { 0, 0, 0xa, }, }
#define AR2C_LP_V4_POS_MSG_INITIALIZER { .bits = { 0, 0, 0xb, }, }
#define AR2C_LP_V4_MIN_CHG_MSG_INITIALIZER { .bits = { 0, 0, 0, 0, 0, 0xe, }, }
#define AR2C_LP_V4_DATE_CHG_MSG_INITIALIZER { .bits = { 0, 0, 0, 0xf, }, }

static __inline__ unsigned char bcd(unsigned char zn)
{
	return (zn > 99) ? 0xff : (((unsigned char)(zn / 10)) << 4)
		+ (unsigned char)(zn % 10);
}

static __inline__ unsigned char from_bcd(unsigned char a)
{
	return (10 * (a >> 4) + (a & 0xf));
}

#if BYTE_ORDER == BIG_ENDIAN
#define MSGTOHL(x)	do { } while (0)
#elif BYTE_ORDER == LITTLE_ENDIAN
#define MSGTOHL(x)	((x) = bswap_32(x))
#else
#error Unknown endianes.
#endif

#endif
