/*
 * ar2c-par-utils - ar2cpar.h 
 * Copyright (C) 2007, 2008  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __AR2CPAR_H
#define __AR2CPAR_H

#include <list.h>
#include <stdio.h>

struct ar2c_param {
	char *name;		/* parameter name */
	char *value;		/* parameter value */
	int prio;		/* parameter priority */
	char *source;		/* parameter source */
	struct list_head list;
};

/* should return error on continue */
#define AR2C_CONFIG_CRIT	0
#define AR2C_CONFIG_ERR		1
#define AR2C_CONFIG_WARNING	2
#define AR2C_CONFIG_NOTICE	3

struct ar2c_config;

/*
 * error handler should return non-zero if configration parsing should
 * be stopped
 */
typedef int (*ar2c_config_error_t)(struct ar2c_config *c, int priority,
		int line, const char *fmt, ...);

struct ar2c_config {
	struct list_head params;

	ar2c_config_error_t error;
};

#ifdef __cplusplus
extern "C" {
#endif

int ar2c_config_init(struct ar2c_config *c);
int ar2c_config_clear(struct ar2c_config *c);
int ar2c_config_destroy(struct ar2c_config *c);
int ar2c_config_set_error_handler(struct ar2c_config *c,
		ar2c_config_error_t handler);

int ar2c_config_load(struct ar2c_config *c, FILE *f);
int ar2c_config_store(struct ar2c_config *c, FILE *f);

struct ar2c_param *ar2c_config_lookup(struct ar2c_config *c,
		const char *name);

const char *__ar2c_config_get(struct ar2c_config *c, const char *name);
int ar2c_config_get(struct ar2c_config *c, const char *name,
		char **value, size_t *len);
int __ar2c_config_set(struct ar2c_config *c, const char *name,
		const char *value);
int ar2c_config_set(struct ar2c_config *c, const char *name,
		const char *value);
int __ar2c_config_set_all(struct ar2c_config *c, const char *name,
		const char *value, const char *source, int prio);
int ar2c_config_set_all(struct ar2c_config *c, const char *name,
		const char *value, const char *source, int prio);
int ar2c_config_set_all_fmt(struct ar2c_config *c, const char *name,
		const char *source, int prio, const char *fmt, ...);
int ar2c_config_rm(struct ar2c_config *c, const char *name);

#ifdef __cplusplus
}
#endif

#define ar2c_config_for_each_param(p, c) \
	list_for_each_entry((p), &(c)->params, list)

#endif
