/*
 * ar2cparproto - ar2cparproto.h
 * Copyright (C) 2008  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * AR-2c Logger Protocol configuration extension
 *
 * This extension is supported by MicroSTER AR-2c/AR-2c+ dataloggers
 * with r2005 or later firmware.
 */

#ifndef __AR2CPARPROTO_H
#define __AR2CPARPROTO_H

#include <inttypes.h>

struct lp_par_pkt {
	uint8_t sync;		/* AR-2c LP synchronization, 0xff */
	uint8_t cmd;		/* AR-2c LP command, 0x0f */
	uint8_t param_id;	/* AR-2c LP extended command, 0x00-0x0f */
	uint8_t plen;		/* payload length */
	uint8_t payload[0];
} __attribute__((packed));

struct lp_par_posvel {
	uint8_t tacho;		/* */
	uint8_t szybowskaz;	/* */
	uint8_t imp;		/* IMP type */
	uint8_t wa2;		/* WA-2 ADC card present */
	uint8_t pommax;		/* POMMAX ADC card present */
	uint8_t dzw_flag;	/* */
	int32_t pkk1;		/* */
	int32_t pkk2;		/* */
} __attribute__((packed));

#define LP_PAR_SYNC		0xff
#define LP_PAR_CMD		0x0f

#define LP_PAR_CRC_LEN		2

#define LP_PARID_NAZWA_KOPALNI	0x01
#define LP_PARID_NAZWA_SZYBU	0x02
#define LP_PARID_BNAMES		0x03
#define LP_PARID_ADESC		0x04
#define LP_PARID_BFUNCTIONS	0x05
#define LP_PARID_NLOGPOM	0x06
#define LP_PARID_POSVEL		0x07
#define LP_PARID_AOFFSETS	0x09
#define LP_PARID_WC8		0x0B
#define LP_PARID_CARD_BUS	0x0C
#define LP_PARID_ADIVS		0x0D

#define LP_PAR_BNAME_LENGTH	2
#define LP_PAR_BNAMES_PER_CARD	8
#define LP_PAR_BINARY_CARD_MAX	23
#define LP_PAR_ADESC_PAGE_MAX	7
#define LP_PAR_ADESC_LEN	42
#define LP_PAR_BFUNCTIONS_COUNT	64
#define LP_PAR_NLOGPOM_COUNT	32
#define LP_PAR_AOFFSETS_COUNT	16
#define LP_PAR_WC8_COUNT	4
#define LP_PAR_CARD_BUS_COUNT	4
#define LP_PAR_ADIVS_COUNT	32

struct lp_par_wc8_config {
	uint8_t id;		/* card id */
	uint16_t base;		/* base address */
	uint8_t dest_offset;	/* destination offset */
	uint8_t length;		/* block length */
} __attribute__((packed));

struct lp_par_card_bus_config {
	uint8_t id;		/* card id */
	uint16_t base;		/* base address */
	uint16_t bin_offset;	/* binary source offset */
	uint16_t bin_dest;	/* binary destination offset */
	uint16_t bin_length;	/* binary block length */
	uint16_t anal_offset;	/* analog source offset */
	uint16_t anal_dest;	/* analog destination offset */
	uint16_t anal_length;	/* analog block length */
	uint16_t hb_offset;	/* HB source offset */
	uint16_t hb_dest;	/* HB destination offset */
	uint16_t hb_length;	/* HB block length */
	uint16_t diag_offset;	/* diagnostic byte offset */
	uint8_t diag_ok;	/* diagnostic valid value */
	uint8_t diag_trfail;	/* transmission failure error value */
	uint8_t chk_value;	/* diagnostic check value */
} __attribute__((packed));

#endif
