/*
 * arbusctl.h
 *
 * Copyright (C) 2012  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __ARBUSCTL_H
#define __ARBUSCTL_H

#if HAVE_INTTYPES_H
#include <inttypes.h>
#elif HAVE_STDINT_H
#include <stdint.h>
#endif

#define PCI_VENDOR_ID_MICROSTER		0xff00	/* non-registered ID*/
#define PCI_DEVICE_ID_MICROSTER_POMMAX2	0x0003

#define ARBUS_CONFIG_REGS	256

struct arbus_dev {
	unsigned int vendor_id;
	unsigned int device_id;

	const char *slot;
	unsigned int devfn;
	unsigned long base_addr[6];

	int valid;
	unsigned char config_regs[ARBUS_CONFIG_REGS];
};

/*
 * The PCI interface treats multi-function devices as independent
 * devices.  The slot/function address of each device is encoded
 * in a single byte as follows:
 *
 *	7:3 = slot
 *	2:0 = function
 */
#define PCI_DEVFN(slot, func)	((((slot) & 0x1f) << 3) | ((func) & 0x07))
#define PCI_SLOT(devfn)		(((devfn) >> 3) & 0x1f)
#define PCI_FUNC(devfn)		((devfn) & 0x07)

typedef uint8_t u8;
typedef uint16_t u16;
typedef uint32_t u32;
typedef int8_t s8;
typedef int16_t s16;
typedef int32_t s32;

int ar2c_readb(unsigned long addr);
int ar2c_readw(unsigned long addr);
int ar2c_readl(unsigned long addr, u32 *x);
int ar2c_writeb(u8 x, unsigned long addr);
int ar2c_writew(u16 x, unsigned long addr);
int ar2c_writel(u32 x, unsigned long addr);
int ar2c_dump_memory(unsigned long addr, u8 *data);

int ar2c_ioread8_copy(u8 *data, unsigned long addr, unsigned int count);
int ar2c_iowrite8_copy(unsigned long addr, const u8 *data,
		unsigned int count);

u8 arbus_read_byte(struct arbus_dev *dev, int pos);
u16 arbus_read_word(struct arbus_dev *dev, int pos);
u32 arbus_read_long(struct arbus_dev *dev, int pos);
int arbus_write_byte(struct arbus_dev *dev, int pos, u8 data);
int arbus_write_word(struct arbus_dev *dev, int pos, u16 data);
int arbus_write_long(struct arbus_dev *dev, int pos, u32 data);

struct ar2c_config;

void pommax3_getconfig(struct arbus_dev *dev, struct ar2c_config *c);
void pommax3_putconfig(struct arbus_dev *dev, struct ar2c_config *c);
void configure_pommax3(struct arbus_dev *dev);
void pommax3_save(struct arbus_dev *dev);
void pommax3_get_data(struct arbus_dev *dev, struct ar2c_config *c);

void dev_error(struct arbus_dev *dev, const char *fmt, ...);

#endif
