/*
 * LP Protocol Debugging Extensions
 * version 0.0
 *
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __LPDE_H
#define __LPDE_H

#if defined(HAVE_INTTYPES_H)
#include <inttypes.h>
#elif defined(HAVE_STDINT_H)
#include <stdint.h>
#endif

/*
 * LPDE_SELECT_PAGE - select top memory page
 */
#define LPDE_SELECT_PAGE	0x8
struct lpde_select_page {
	uint8_t page;		/* page number */
} __attribute__((packed));

/*
 * LPDE_DUMP_MEMORY - dump 256 byte memory region
 */
#define LPDE_DUMP_MEMORY	0x9
struct lpde_dump_memory {
	uint16_t offset;	/* address offset, big-endian */
	uint16_t segment;	/* address segment, big-endian */
} __attribute__((packed));

struct lpde_dump_memory_r {
	uint8_t data[256];
} __attribute__((packed));

/*
 * LPDE_DUMP_TOP_MEMORY - dump 256 byte memory region from 0xa000 segment
 */
#define LPDE_DUMP_TOP_MEMORY	0xa
struct lpde_dump_top_memory {
	uint16_t offset;	/* address offset, big-endian */
} __attribute__((packed));

struct lpde_dump_top_memory_r {
	uint8_t data[256];
} __attribute__((packed));

/*
 * LPDE_DUMP_BOTTOM_MEMORY - dump 256 byte memory region from 0xb000 segment
 */
#define LPDE_DUMP_BOTTOM_MEMORY	0xb
struct lpde_dump_bottom_memory {
	uint16_t offset;	/* address offset, big-endian */
} __attribute__((packed));

struct lpde_dump_bottom_memory_r {
	uint8_t data[256];
} __attribute__((packed));

/*
 * LPDE_READB - read byte from memory
 *
 * segment of last dump operation is used
 */
#define LPDE_READB		0xc
struct lpde_readb {
	uint16_t offset;	/* address offset, big-endian */
} __attribute__((packed));

struct lpde_readb_r {
	uint16_t offset;	/* address offset, big-endian */
	uint8_t value;
} __attribute__((packed));

/*
 * LPDE_WRITEB - write byte to memory
 *
 * address of last LPDE_READB command is used
 */
#define LPDE_WRITEB		0xd
struct lpde_writeb {
	uint8_t value;		/* value to write */
} __attribute__((packed));

struct lpde_writeb_r {
	uint16_t next_offset;	/* offset of next byte, big-endian */
	uint8_t next_value;	/* value of next byte */
} __attribute__((packed));

#endif
