/*
 * ar2c-lp-utils - t_read.h
 * 	version 0.0.1
 * 
 * Copyright (C) 2005  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef TREAD_H_INCLUDED
#define TREAD_H_INCLUDED

#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <stdio.h>
#include <sys/poll.h>

static __inline__ ssize_t t_read_nb(int fd, void *_buff, size_t size,
		int timeout)
{
	char *buff = (char *) _buff;
	struct pollfd pfd;
	int ret;

	pfd.fd = fd;
	pfd.events = POLLIN;
	ret = poll(&pfd, 1, timeout);
	if (ret < 0)
		return -errno;
	if (ret == 0)
		return 0;	/* timeout */
	if ((pfd.revents & POLLIN) != POLLIN)
		return -1;	/* error */
	ret = read(fd, buff, size);
	if (ret < 0)
		return -errno;
	return ret;
}

static __inline__ ssize_t t_read(int fd, void *_buff, size_t size, int timeout)
{
	char *buff = (char *) _buff;
	int len = size, readed;
	struct pollfd pfd;
	int ret;

	pfd.fd = fd;
	pfd.events = POLLIN;

	while (len > 0) {
		ret = poll(&pfd, 1, timeout);
		if (ret < 0)
			return -errno;
		if (ret == 0)
			return size - len;	/* timeout */
		if ((pfd.revents & POLLIN) != POLLIN)
			return -1;		/* error */
		readed = read(fd, buff, len);
		if (readed <= 0)
			return size - len;
		buff += readed;
		len -= readed;
	}
	return size;
}

#endif
